<?php

/**
 * \file Coordsyss.class.inc
 * \class Coordsyss
 *
 * \author WAB <support.wab@veremes.com>.
 *
 * 	\brief This file contains the Coordsyss php class
 *
 * This class defines Rest Api to Vitis Coordsyss
 * 
 */
require_once __DIR__ . '/ExtractionExtraction.class.inc';
require_once 'Coordsys.class.inc';


class Coordsyss extends ExtractionExtraction {
    /**
     * @SWG\Definition(
     *   definition="/coordsyss",
     *   allOf={
     *     @SWG\Schema(ref="#/definitions/coordsyss")
     *   }
     * )
     * * @SWG\Tag(
     *   name="Coordsyss",
     *   description="Operations about Coordsyss"
     * )
     */

    /**
     * construct
     * @param type $aPath url of the request
     * @param type $aValues parameters of the request
     * @param type $properties properties
     * @param type $bShortcut false to reinit variables
     * @param type $oConnection connection object
     */
    function __construct($aPath, $aValues, $properties, $bShortcut = false, $oConnection = false){
        parent::__construct($aPath, $aValues, $properties, $bShortcut, $oConnection);
    }

/**
     * @SWG\Get(path="/coordsyss",
     *   tags={"Coordsyss"},
     *   summary="Get Coordsyss",
     *   description="Request to get Coordsyss",
     *   operationId="GET",
     *   produces={"application/xml", "application/json", "application/x-vm-json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="order_by",
     *     in="query",
     *     description="list of ordering fields",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="sort_order",
     *     in="query",
     *     description="sort_order",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="limit",
     *     in="query",
     *     description="number of element",
     *     required=false,
     *     type="integer",
     *     format="int32"
     *   ),
     * @SWG\Parameter(
     *     name="offset",
     *     in="query",
     *     description="index of first element",
     *     required=false,
     *     type="string",
     *     format="int32"
     *   ),
     * @SWG\Parameter(
     *     name="attributs",
     *     in="query",
     *     description="list of attributs",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="filter",
     *     in="query",
     *     description="filter results",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="distinct",
     *     in="query",
     *     description="delete duplicates",
     *     required=false,
     *     type="boolean"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="coordsys Response",
     *         @SWG\Schema(ref="#/definitions/coordsyss")
     *     )
     *  )
     */

    /**
     * get Coordsyss
     * @return  Coordsyss
     */
    function GET() {
        $aReturn = $this->genericGet($this->aProperties['schema_extraction'], "rt_coordsys", "coordsys_id");
        $sMessage = $aReturn['sMessage'];
        return $sMessage;
    }

/**
     * @SWG\Post(path="/coordsyss",
     *   tags={"Coordsyss"},
     *   summary="Add coordsys",
     *   description="Request to add Coordsyss",
     *   operationId="POST",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="coordsys Response",
     *         @SWG\Schema(ref="#/definitions/coordsyss")
     *     )
     *  )
     */

    /**
     * insert coordsys
     * @return id of the coordsys created
     */
    function POST() {
        $aReturn = $this->genericPost($this->aProperties['schema_extraction'], 'rt_coordsys', '', 'coordsys_id');
        return $aReturn['sMessage'];
    }

/**
     * @SWG\Put(path="/coordsyss/{id_coordsys}",
     *   tags={"Coordsyss"},
     *   summary="update Coordsyss",
     *   description="Request to update Coordsyss",
     *   operationId="PUT",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="Coordsys token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="id_coordsys",
     *     in="path",
     *     description="id of the Coordsyss",
     *     required=true,
     *     type="integer",
     *     format = "int32"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/coordsyss")
     *     )
     *  )
     */

    /**
     * update coordsyss
     * @return id of coordsyss updated or error object if a coordsyss is not updated
     */
    function PUT() {
        $aReturn = $this->genericPut($this->aProperties['schema_extraction'], 'rt_coordsys', 'coordsys_id');
        
        return $aReturn['sMessage'];
    }

/**
     * @SWG\Delete(path="/coordsyss",
     *   tags={"Coordsyss"},
     *   summary="delete Coordsyss",
     *   description="Request to delete Coordsyss",
     *   operationId="DELETE",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="idList",
     *     in="query",
     *     description="id of the coordsys",
     *     required=true,
     *     type="string"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="coordsys Response",
     *         @SWG\Schema(ref="#/definitions/coordsyss")
     *     )
     *  )
     */
    /**
     * @SWG\Delete(path="/coordsyss/{id_coordsys}",
     *   tags={"Coordsyss"},
     *   summary="delete Coordsyss",
     *   description="Request to delete Coordsyss",
     *   operationId="DELETE",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="Coordsys token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="id_coordsys",
     *     in="path",
     *     description="id of the Coordsyss",
     *     required=true,
     *     type="integer",
     *     format = "int32"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/coordsyss")
     *     )
     *  )
     */

    /**
     * delete coordsyss
     * @return id of coordsyss deleted or error object if a coordsyss is not deleted
     */
    function DELETE() {
        $aReturn = $this->genericDelete($this->aProperties['schema_extraction'], 'rt_coordsys', 'coordsys_id');
        return $aReturn['sMessage'];
    }

	
}

?>