<?php

require_once __DIR__ . '/ExtractionExtraction.class.inc';
require_once $this->aProperties['vas_home'] . '/rest/class/vitis_lib/Connection.class.inc';

/**
 * \file Extraction.class.inc
 * \class Extraction
 *
 * \author WAB <support.wab@veremes.com>.
 *
 *     \brief This file contains the Extraction php class
 *
 * This class defines operation for one Extraction
 *
 */
class Extraction extends ExtractionExtraction
{

    public $oError;

    /**
     * construct
     * @param type $aPath url of the request
     * @param type $aValues parameters of the request
     * @param type $properties properties
     * @param type $bShortcut false to reinit variables
     * @param type $oConnection connection object
     */
    public function __construct($aPath, $aValues, $properties, $bShortcut = false, $oConnection = false)
    {
        parent::__construct($aPath, $aValues, $properties, $bShortcut, $oConnection);
    }

/**
 * @SWG\Get(path="/extractions/{id_extraction}",
 *   tags={"Extractions"},
 *   summary="Get Extraction",
 *   description="Request to get Extraction by id",
 *   operationId="GET",
 *   produces={"application/xml", "application/json"},
 *   @SWG\Parameter(
 *     name="token",
 *     in="query",
 *     description="user token",
 *     required=true,
 *     type="string"
 *   ),
 * @SWG\Parameter(
 *     name="id_extraction",
 *     in="path",
 *     description="id_extraction",
 *     required=true,
 *     type="integer",
 *   format="int32"
 *   ),
 * @SWG\Parameter(
 *     name="attributs",
 *     in="query",
 *     description="list of attributs",
 *     required=false,
 *     type="string"
 *   ),
 *   @SWG\Response(
 *         response=200,
 *         description="Extraction Response",
 *         @SWG\Schema(ref="#/definitions/extractions")
 *     )
 *  )
 */

    /**
     * get informations about mode
     */
    public function GET()
    {
        require $this->sRessourcesFile;
        $this->aFields = $this->getFields($this->aProperties['schema_extraction'], "v_extraction", "bo_id");
        // Url vers l'acte d'engagement.
        if (!empty($this->aFields['ae'])) {
            $this->aFields['ae'] = $this->aProperties['web_server_name'] . '/' . $this->aProperties['ws_data_alias'] . '/extraction/extraction_extraction_extraction/documents/' . $this->aValues["my_vitis_id"] . '/ae/' . $this->aFields['ae'];
        }

        // Date au format Français.
        if ($this->aProperties["language"] == 'fr') {
            $oDate                          = new DateTime($this->aFields['creation_date']);
            $this->aFields['creation_date'] = $oDate->format('d/m/Y');
        }

        // Libellés du champ "Commune".
        if (!empty($this->aFields['id_com'])) {
            //On donne les paramÃ¨tres Ã  remplacer dans la requÃªte
            $aSQLParams = array(
                'id_com' => array('value' => $this->aFields['id_com'], 'type' => 'group'),
            );

            $sSql = $aSql['getCommuneLabel'];

            $oPDOresult = $this->oConnection->oBd->executeWithParams($sSql, $aSQLParams);
            if (!$this->oConnection->oBd->enErreur()) {
                while ($aLine = $this->oConnection->oBd->ligneSuivante($oPDOresult)) {
                    $aCommune[] = $aLine['nom_com'];
                }
                $this->aFields['id_com_label'] = implode(',', $aCommune);
            }
        }

        // LibellÃ©s du champ "Conseil de Territoire".
        if (!empty($this->aFields['id_conseil'])) {

            $aSQLParams = array(
                'id_conseil' => array('value' => $this->aFields['id_conseil'], 'type' => 'group'),
            );
            //$sSql = str_replace('[id_conseil]', str_replace('|', ',', $this->aFields['id_conseil']), $aSql['getConseilLabel']);
            $aConseil = [];
            $sSql       = $aSql['getConseilLabel'];
            $oPDOresult = $this->oConnection->oBd->executeWithParams($sSql, $aSQLParams);
            if (!$this->oConnection->oBd->enErreur()) {
                while ($aLine = $this->oConnection->oBd->ligneSuivante($oPDOresult)) {
                    $aConseil[] = $aLine['nom_epci'];
                }
                if (is_array($aConseil)) {
                    $this->aFields['id_conseil_label'] = implode(',', $aConseil);
                } else {
                    $this->aFields['id_conseil_label'][0] = $aConseil;
                }
            }
        }

        // Nom de l'Ã©tat de l'extraction.
        error_log(print_r($this->aFields['status'], true));
        $this->aFields['status_name'] = $this->aFields['status'];
    }

/**
 * delete a Point_situation
 */
    public function DELETE()
    {
        $this->oConnection->oBd->delete($this->aProperties['schema_extraction'], 'extraction', 'bo_id', $this->aValues["my_vitis_id"], 'integer');
        if ($this->oConnection->oBd->enErreur()) {
            $this->oError = new Error(1, $this->oConnection->oBd->getBDMessage());
        } else {
            $this->aFields["bo_id"] = $this->aValues["my_vitis_id"];
        }
    }

}
