<?php

/**
 * \file Extractions.class.inc
 * \class Extractions
 *
 * \author WAB <support.wab@veremes.com>.
 *
 *     \brief This file contains the Extractions php class
 *
 * This class defines Rest Api to Vitis Extractions
 *
 */
require_once __DIR__ . '/ExtractionExtraction.class.inc';
require_once 'Extraction.class.inc';
require_once $this->aProperties['vas_home'] . '/rest/class/wab_lib/BusinessObject.class.inc';

class Extractions extends ExtractionExtraction
{

    protected $iIdExtraction;
    /**
     * @SWG\Definition(
     *   definition="/extractions",
     *   allOf={
     *     @SWG\Schema(ref="#/definitions/extractions")
     *   }
     * )
     * * @SWG\Tag(
     *   name="Extractions",
     *   description="Operations about Extractions"
     * )
     */

    /**
     * construct
     * @param type $aPath url of the request
     * @param type $aValues parameters of the request
     * @param type $properties properties
     * @param type $bShortcut false to reinit variables
     * @param type $oConnection connection object
     */
    public function __construct($aPath, $aValues, $properties, $bShortcut = false, $oConnection = false)
    {
        parent::__construct($aPath, $aValues, $properties, $bShortcut, $oConnection);
    }

/**
 * @SWG\Get(path="/extractions",
 *   tags={"Extractions"},
 *   summary="Get Extractions",
 *   description="Request to get Extractions",
 *   operationId="GET",
 *   produces={"application/xml", "application/json", "application/x-vm-json"},
 *   @SWG\Parameter(
 *     name="token",
 *     in="query",
 *     description="user token",
 *     required=true,
 *     type="string"
 *   ),
 * @SWG\Parameter(
 *     name="order_by",
 *     in="query",
 *     description="list of ordering fields",
 *     required=false,
 *     type="string"
 *   ),
 * @SWG\Parameter(
 *     name="sort_order",
 *     in="query",
 *     description="sort_order",
 *     required=false,
 *     type="string"
 *   ),
 * @SWG\Parameter(
 *     name="limit",
 *     in="query",
 *     description="number of element",
 *     required=false,
 *     type="integer",
 *     format="int32"
 *   ),
 * @SWG\Parameter(
 *     name="offset",
 *     in="query",
 *     description="index of first element",
 *     required=false,
 *     type="string",
 *     format="int32"
 *   ),
 * @SWG\Parameter(
 *     name="attributs",
 *     in="query",
 *     description="list of attributs",
 *     required=false,
 *     type="string"
 *   ),
 * @SWG\Parameter(
 *     name="filter",
 *     in="query",
 *     description="filter results",
 *     required=false,
 *     type="string"
 *   ),
 * @SWG\Parameter(
 *     name="distinct",
 *     in="query",
 *     description="delete duplicates",
 *     required=false,
 *     type="boolean"
 *   ),
 *   @SWG\Response(
 *         response=200,
 *         description="extraction Response",
 *         @SWG\Schema(ref="#/definitions/extractions")
 *     )
 *  )
 */
    /**
     * @SWG\Get(path="/extractions/Layers",
     *   tags={"Extractions"},
     *   summary="Get extraction layers",
     *   description="Request to get the layers of the extractions",
     *   operationId="GET",
     *   produces={"application/xml", "application/json", "application/x-vm-json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/extractions")
     *     )
     *  )
     */

    /**
     * get Extractions
     * @return  Extractions
     */
    public function GET()
    {
        if (!empty($this->aPath[2]) && $this->aPath[2] == "Layers") {
            $sMessage = $this->getLayers();
        } else {
            $aReturn  = $this->genericGet($this->aProperties['schema_extraction'], "v_extraction", "bo_id");
            $sMessage = $aReturn['sMessage'];
        }
        return $sMessage;
    }

/**
 * @SWG\Post(path="/extractions",
 *   tags={"Extractions"},
 *   summary="Add extraction",
 *   description="Request to add Extractions",
 *   operationId="POST",
 *   produces={"application/xml", "application/json"},
 *   @SWG\Parameter(
 *     name="token",
 *     in="query",
 *     description="user token",
 *     required=true,
 *     type="string"
 *   ),
 *   @SWG\Response(
 *         response=200,
 *         description="extraction Response",
 *         @SWG\Schema(ref="#/definitions/extractions")
 *     )
 *  )
 */

    /**
     * insert extraction
     * @return id of the extraction created
     */
    public function POST()
    {
        $aReturn                = $this->genericPost($this->aProperties['schema_extraction'], 'extraction', $this->aProperties['schema_extraction'] . '.bo_id_extraction_seq', 'bo_id');
        $this->aValues["bo_id"] = $this->aValues['my_vitis_id'];
        eval('$this->' . $this->aValues['tr_status_method'] . '();');
        return $aReturn['sMessage'];
    }

/**
 * @SWG\Put(path="/extractions/{id_extraction}",
 *   tags={"Extractions"},
 *   summary="update Extractions",
 *   description="Request to update Extractions",
 *   operationId="PUT",
 *   produces={"application/xml", "application/json"},
 *   @SWG\Parameter(
 *     name="token",
 *     in="query",
 *     description="Extraction token",
 *     required=true,
 *     type="string"
 *   ),
 * * @SWG\Parameter(
 *     name="id_extraction",
 *     in="path",
 *     description="id of the Extractions",
 *     required=true,
 *     type="integer",
 *     format = "int32"
 *   ),
 * @SWG\Response(
 *         response=200,
 *         description="Poprerties Response",
 *         @SWG\Schema(ref="#/definitions/extractions")
 *     )
 *  )
 */

    /**
     * update extractions
     * @return id of extractions updated or error object if a extractions is not updated
     */
    public function PUT()
    {
        // Upload l'acte d'engagement.
        $this->uploadDocument('ae', 'extraction_extraction_extraction');
        //
        $aReturn                = $this->genericPut($this->aProperties['schema_extraction'], 'extraction', 'bo_id');
        $this->aValues["bo_id"] = $this->aValues['my_vitis_id'];
        // Méthode de transition.
        if (!empty($this->aValues['tr_status_method'])) {
            eval('$this->' . $this->aValues['tr_status_method'] . '();');
        }

        //On vérifie que le fichier existe et ne soit pas vide
        if (!empty($this->aValues['input_deposit_file'])) {
            //On va déplacer le fichier dans un dossier temporaire
            $aXmlRacineAttribute['status'] = 1;
            $sMessage                      = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], true, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
            $sDirPath                      = $this->aProperties["upload_dir"] . "/" . getUniqRandomId();
            while (is_dir($sDirPath)) {
                $sDirPath = $this->aProperties["upload_dir"] . "/" . getUniqRandomId();
            }
            if (!mkdir($sDirPath)) {
                return false;
            }
            // Sauvegarde le nouveau fichier.
            $fp = fopen($sDirPath . '/' . utf8_decode($this->aValues["input_deposit_name"]), "w");
            fwrite($fp, $this->aValues["input_deposit_file"]);
            fclose($fp);
            //Demande GTF
            $this->oBO->insertOrder('EM0160AYYTK48', 'sourceDir=' . end(explode("/", $sDirPath)) . '/' . $this->aValues["input_deposit_name"] . '|sourceMotif=*.shp');
        }
        return $aReturn['sMessage'];
    }

/**
 * @SWG\Delete(path="/extractions",
 *   tags={"Extractions"},
 *   summary="delete Extractions",
 *   description="Request to delete Extractions",
 *   operationId="DELETE",
 *   produces={"application/xml", "application/json"},
 *   @SWG\Parameter(
 *     name="token",
 *     in="query",
 *     description="token",
 *     required=true,
 *     type="string"
 *   ),
 * * @SWG\Parameter(
 *     name="idList",
 *     in="query",
 *     description="id of the extraction",
 *     required=true,
 *     type="string"
 *   ),
 * @SWG\Response(
 *         response=200,
 *         description="extraction Response",
 *         @SWG\Schema(ref="#/definitions/extractions")
 *     )
 *  )
 */
    /**
     * @SWG\Delete(path="/extractions/{id_extraction}",
     *   tags={"Extractions"},
     *   summary="delete Extractions",
     *   description="Request to delete Extractions",
     *   operationId="DELETE",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="Extraction token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="id_extraction",
     *     in="path",
     *     description="id of the Extractions",
     *     required=true,
     *     type="integer",
     *     format = "int32"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/extractions")
     *     )
     *  )
     */

    /**
     * delete extractions
     * @return id of extractions deleted or error object if a extractions is not deleted
     */
    public function DELETE()
    {
        $aReturn = $this->genericDelete($this->aProperties['schema_extraction'], 'extraction', 'bo_id');
        return $aReturn['sMessage'];
    }

    /**
     * Liste des couches.
     */
    public function getLayers()
    {
        require $this->sRessourcesFile;
        //On décode le json récupéré via le filtre
        if (!empty($this->aValues["filter"])) {
            $decode = $this->decodeJSONFilter($this->aValues["filter"], $this->aProperties['schema_extraction'], $this->aValues["table"]);
        }

        if (is_object($this->aValues) || is_array($this->aValues)) {
            $sJSONFilter = json_encode($this->aValues);
        }

        if (!empty($sJSONFilter)) {
            $aJSONFilter = json_decode($sJSONFilter, true);

            if (!empty($aJSONFilter["filter"])) {
                if (is_object($aJSONFilter["filter"]) || is_array($aJSONFilter["filter"])) {
                    $sJSONFilter = json_encode($aJSONFilter["filter"]);
                }
            }
        }
        $aSQLParams = array(
            'schema_framework' => array('value' => $this->aProperties['schema_framework'], 'type' => 'schema_name'),
            'user_id'          => array('value' => $_SESSION['ses_user_id'], 'type' => 'string'),
        );

        $sSql       = $aSql['getUserGroups'];
        $oPDOresult = $this->oConnection->oBd->executeWithParams($sSql, $aSQLParams);
        if (!$this->oConnection->oBd->enErreur()) {
            $aResult = $this->oConnection->oBd->getResultTableAssoc($oPDOresult);
            foreach ($aResult as $aUserGroup) {
                $aUserGroups[] = $aUserGroup['name'];
            }

            if (!empty($aValues["filter"])) {
                $aSQLParams = array(
                    'schema_extraction' => array('value' => $this->aProperties['schema_extraction'], 'type' => 'schema_name'),
                );

                $sSql = $aSql['getCtId'];
                //On donne la condition
                if ($decode["request"]) {
                    $sSql .= " WHERE ";
                    $sSql .= $decode["request"];
                    $sSql .= 'ORDER BY ct_id;';
                }
                //error_log(print_r($sSql, true));
                $merge = array_merge($aSQLParams, $decode["params"]);

                $oPDOresult = $this->oConnection->oBd->executeWithParams($sSql, $merge);

                $aCtResult = $this->oConnection->oBd->getResultTableAssoc($oPDOresult);
            }
            // Restriction de l'utilisateur suivant ses groupes.
            if (in_array('interne', $aUserGroups)) {
                $sSqlParams = " ORDER BY category, layer_id;";
            } else if (in_array('restreint', $aUserGroups)) {
                $sSqlParams = " AND restriction <> 'interne' ORDER BY category, layer_id;";
            } else {
                $sSqlParams = " AND restriction = 'public' ORDER BY category, layer_id;";
            }

            // Liste des cartes.
            $this->aFields['treeview_layers'] = array();

            //Requête pour récupérer les couches
            if (!empty($aValues["filter"])) {
                $sSql = $aSql['getRtLayerWithCtId'] . $sSqlParams;
            } else {
                $sSql = $aSql['getRtLayer'] . $sSqlParams;
            }
            $ct_id;
            if (!empty($aCtResult)) {
                for ($i = 0; $i < sizeof($aCtResult); $i++) {
                    $ct_id .= $aCtResult[$i]["ct_id"] . "|";
                }
                //On retire la dernière pipe qui ne sert à rien
                $ct_idToSend = substr($ct_id, 0, -1);
            } else {
                //On donne un id qui n'existe pas pour éviter une erreur dans la requête SQL
                $ct_idToSend = -1;
            }
            //error_log(print_r($aCtResult[$i]["ct_id"], true));
            $aSQLParams = array(
                'schema_extraction' => array('value' => $this->aProperties['schema_extraction'], 'type' => 'schema_name'),
                'aCtId'             => array('value' => $ct_idToSend, 'type' => 'group'),
            );
            $oPDOresult = $this->oConnection->oBd->executeWithParams($sSql, $aSQLParams);
            if (!$this->oConnection->oBd->enErreur()) {
                $aRtLayer = $this->oConnection->oBd->getResultTableAssoc($oPDOresult);
                $aLayers  = array();
                if (!empty($aRtLayer)) {
                    foreach ($aRtLayer as $aLayer) {

                        $aLayers[$aLayer['category']][] = array("text" => $aLayer['layer'], "value" => $aLayer['layer_id'], "mutualized" => $aLayer['mutualized'], "statistique" => $aLayer['statistique']);

                    }
                }
                $this->aFields['layers']       = $aLayers;
                $aXmlRacineAttribute['status'] = 1;
                $sMessage                      = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], true, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
            } else {
                $aXmlRacineAttribute['status'] = 0;
                $this->aFields['error']        = "Request error.";
                $sMessage                      = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], true, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
            }
        } else {
            $aXmlRacineAttribute['status'] = 0;
            $this->aFields['error']        = "Request error.";
            $sMessage                      = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], true, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
        }
        //error_log(print_r($sMessage, true));
        return $sMessage;
    }

    /**
     * Transition Method trNouveau
     */
    public function trNouveau()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);

        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Nouveau");

    }

    /**
     * Transition Method trGenerer
     */
    public function trGenerer()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);

        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "AE Généré");

    }

    /**
     * Transition Method trResoumettre
     */
    public function trResoumettre()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);

        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "AE Généré");
//$this->sendmail(9904);
        $this->sendmail(171);
    }

    /**
     * Transition Method trSoumettre
     */
    public function trSoumettre()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);

        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "AE Soumis");
        //$this->oBO->sendmail(39);
        $this->oBO->sendmail(174);
    }

    /**
     * Transition Method trDemander
     */
    public function trDemander()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        $this->oBO->insertOrder('832HZ8J2PTCSG', 'bo_id=' . $this->aValues['bo_id']);
        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "AE Demandé");

    }

    /**
     * Transition Method trReception
     */
    public function trReception()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);

        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Dépôt terminé");

    }

    /**
     * Transition Method trDeposer
     */
    public function trDeposer()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Fichier déposé");

    }

    /**
     * Transition Method trAutoriser
     */
    public function trAutoriser()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        $this->oBO->insertOrder('8BCP8WGIOW4KO', 'bo_id=' . $this->aValues['bo_id']);
        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Autorisée");

    }

    /**
     * Transition Method trAccepter
     */
    public function trAccepter()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);

        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Fichier accepté");
        //$this->oBO->sendmail(id_du_mail_pour_dire_que_le_fichier_est_accepté);
        $this->trReception();

    }

    /**
     * Transition Method trInvalider
     */
    public function trInvalider()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);

        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Fichier invalidé");
        //$this->oBO->sendmail(id_du_mail_pour_dire_que_le_fichier_est_invalidé);
        $this->trRetour();

    }

    /**
     * Transition Method trRetour
     */
    public function trRetour()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);

        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Extraction terminée");

    }

    /**
     * Transition Method trTerminerExtraction
     */
    public function trTerminerExtraction()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);
        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Extraction terminée");

    }

    /**
     * Transition Method trRefuser
     */
    public function trRefuser()
    {
        $this->oBO = new WabBusinessObject($this->oConnection->oBd, array(), $this->aValues, $this->aProperties, 'extraction', $this->aValues['bo_id']);

        // Check si la table existe si oui change l'état
        $this->oBO->wabSetStatus($this->aProperties["schema_extraction"], "extraction", "bo_id", $this->aValues["bo_id"], "Refusée");
//$this->sendmail(9902);
        $this->oBO->sendmail(175);
    }

}
